#Link CPs - Surface Variables (Iberian Peninsula and Catalunya)
#Plotting spatial patterns (anom)
library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
#library(matrixStats)
#library(metR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "/esarchive/scratch/molmo/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

do <- c("D1","D2","D3")
#######################---tasmax---####################################

for(d in 1:3){
  dom <- do[d]
  
  for(k in c(3,4,5,7)){
    
#Łoad observational data
nc <- nc_open("era5_tasmax_5021_anom.nc")
nc_var <- ncvar_get(nc,"tasmax")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

domain1 <- list(lon <- which(nc_lon > -15 & nc_lon < 15),
                lat <- which(nc_lat > 32 & nc_lat < 48)) #red domain D1 #Spain

domain2 <- list(lon <- which(nc_lon > -10 & nc_lon < 25),
                lat <- which(nc_lat > 33 & nc_lat < 45)) #red domain D2 PEP Mediterranean

domain3 <- list(lon <- which(nc_lon > -35 & nc_lon < 35),
                lat <- which(nc_lat > 20 & nc_lat < 70)) #red domain D3 #Europe

#Load clustering information
#domain <- domain1 ; k=3; dom = "D1" ; x1 = -15 ; x2 = 15 ; y1 = 32 ; y2 = 48
domain <- domain1 #; k=7 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
#domain <- domain1 ; k=7; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70

nc_var <- nc_var[domain[[1]],domain[[2]],which(format(calendario,"%Y")%in%c(1950:2021))] ; gc()

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()

load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]

#Preparing data
longitude=rep(nc_lon[domain[[1]]],length(domain[[2]]))
latitude=rep(nc_lat[domain[[2]]],each=length(domain[[1]]))

Z <- list() 

for(cp in 1:k){
  Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
}

datitos <- data.frame(Longitude=rep(longitude,k),
                      Latitude=rep(latitude,k),
                      Z = unlist(Z),
                      CP = rep(1:k,each=length(unlist(Z))/k))
datitos$CP <- factor(datitos$CP,levels=c(1:k))


p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-4,4,by=2),limits=c(-4,4),
                       colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("TX Anom Patterns") 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill=NA,alpha=0.003)
ggsave(p,filename=paste("SpatialPatterns_Ward_Surface_TX_Anom_",dom,"_Iberia_",k,"CPs_ERA5.png",sep=""),device="png",
       dpi=600,width=2*k,height = 2)

p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-4,4,by=2),limits=c(-4,4),
                       colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("TX Anom Patterns") 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
ggsave(p,filename=paste("SpatialPatterns_Ward_Surface_TX_Anom_",dom,"_Catalunya_",k,"CPs_ERA5.png",sep=""),device="png",
       dpi=600,width=2*k,height = 2)

####################################################################################
#######################---tasmin---####################################
rm(datitos,nc,p,var,Z) ; gc()

#Łoad observational data
nc <- nc_open("era5_tasmin_5021_anom.nc")
nc_var <- ncvar_get(nc,"tasmin")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

domain1 <- list(lon <- which(nc_lon > -15 & nc_lon < 15),
                lat <- which(nc_lat > 32 & nc_lat < 48)) #red domain D1 #Spain

domain2 <- list(lon <- which(nc_lon > -10 & nc_lon < 25),
                lat <- which(nc_lat > 33 & nc_lat < 45)) #red domain D2 PEP Mediterranean

domain3 <- list(lon <- which(nc_lon > -35 & nc_lon < 35),
                lat <- which(nc_lat > 20 & nc_lat < 70)) #red domain D3 #Europe

#Load clustering information
#domain <- domain1 ; k=3; dom = "D1" ; x1 = -15 ; x2 = 15 ; y1 = 32 ; y2 = 48
domain <- domain1 #; k=7 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
#domain <- domain1 ; k=7; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70

nc_var <- nc_var[domain[[1]],domain[[2]],which(format(calendario,"%Y")%in%c(1950:2021))] ; gc()

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()

load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]

#Preparing data
longitude=rep(nc_lon[domain[[1]]],length(domain[[2]]))
latitude=rep(nc_lat[domain[[2]]],each=length(domain[[1]]))

Z <- list() 

for(cp in 1:k){
  Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
}

datitos <- data.frame(Longitude=rep(longitude,k),
                      Latitude=rep(latitude,k),
                      Z = unlist(Z),
                      CP = rep(1:k,each=length(unlist(Z))/k))
datitos$CP <- factor(datitos$CP,levels=c(1:k))


p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-4,4,by=2),limits=c(-4,4),
                       colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("TN Anom Patterns") 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill=NA,alpha=0.003)
ggsave(p,filename=paste("SpatialPatterns_Ward_Surface_TN_Anom_",dom,"_Iberia_",k,"CPs_ERA5.png",sep=""),device="png",
       dpi=600,width=2*k,height = 6)

p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-4,4,by=2),limits=c(-4,4),
                       colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("TN Anom Patterns") 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
ggsave(p,filename=paste("SpatialPatterns_Ward_Surface_TN_Anom_",dom,"_Catalunya_",k,"CPs_ERA5.png",sep=""),device="png",
       dpi=600,width=2*k,height = 2)

#################---precip---##############################################
rm(datitos,nc,p,var,Z) ; gc()

#Łoad observational data
nc <- nc_open("era5_prlr_5021_anom.nc")
nc_var <- ncvar_get(nc,"prlr")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

domain1 <- list(lon <- which(nc_lon > -15 & nc_lon < 15),
                lat <- which(nc_lat > 32 & nc_lat < 48)) #red domain D1 #Spain

domain2 <- list(lon <- which(nc_lon > -10 & nc_lon < 25),
                lat <- which(nc_lat > 33 & nc_lat < 45)) #red domain D2 PEP Mediterranean

domain3 <- list(lon <- which(nc_lon > -35 & nc_lon < 35),
                lat <- which(nc_lat > 20 & nc_lat < 70)) #red domain D3 #Europe

#Load clustering information
#domain <- domain1 ; k=3; dom = "D1" ; x1 = -15 ; x2 = 15 ; y1 = 32 ; y2 = 48
domain <- domain1 #; k=7 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
#domain <- domain1 ; k=7; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70

nc_var <- nc_var[domain[[1]],domain[[2]],which(format(calendario,"%Y")%in%c(1950:2021))] ; gc()

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()
#var[which(var<=1)] <- NA

load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]

#Preparing data
longitude=rep(nc_lon[domain[[1]]],length(domain[[2]]))
latitude=rep(nc_lat[domain[[2]]],each=length(domain[[1]]))

Z <- list() 

for(cp in 1:k){
  Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
}

datitos <- data.frame(Longitude=rep(longitude,k),
                      Latitude=rep(latitude,k),
                      Z = unlist(Z),
                      CP = rep(1:k,each=length(unlist(Z))/k))
datitos$CP <- factor(datitos$CP,levels=c(1:k))


p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("Precipitation (mm)"),breaks=seq(-4,4,by=2),limits=c(-4,4),
                       colours=brewer.pal(9,"BrBG"),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("PR Anom Patterns") 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
p <- p + geom_rect(aes(xmin=0,xmax=3,ymin=40,ymax=43),col="black",fill="grey",alpha=0.003)
ggsave(p,filename=paste("SpatialPatterns_Ward_Surface_PR_Anom_",dom,"_Iberia_",k,"CPs_ERA5.png",sep=""),device="png",
       dpi=600,width=6,height = 6)

p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,ncol=k)+ 
  scale_fill_gradientn(name=expression("Precipitation (mm)"),breaks=seq(-4,4,by=2),limits=c(-4,4),
                       colours=brewer.pal(9,"BrBG"),aesthetics=c("fill"),
                       na.value = "white") +
  coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
  xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("PR Anom Patterns") 
#p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
ggsave(p,filename=paste("SpatialPatterns_Ward_Surface_PR_Anom_",dom,"_Catalunya_",k,"CPs_ERA5.png",sep=""),device="png",
       dpi=600,width=2*k,height = 2)
rm(p,datitos,nc,var); gc()
  }
}
